package net.nevet5gi.buzzbot;

import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import net.nevet5gi.buzzbot.commands.utils.CommandManager;
import net.nevet5gi.buzzbot.modules.BuzzyModule;
import net.nevet5gi.buzzbot.modules.ModuleLoader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.security.auth.login.LoginException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.EnumSet;

public class Bot {
    //TODO Make shutdown sequence
    public static JDA jda;
    private static final Logger LOGGER = LoggerFactory.getLogger(Bot.class);
    public static final ArrayList<ListenerAdapter> PLUGINS = new ArrayList<>();
    private static ArrayList<BuzzyModule> modules;

    public static void main(String[] args) {
        Config.loadConfig();
        try {
            modules = ModuleLoader.loadModules();
        } catch (IOException | ClassNotFoundException | NoSuchMethodException | InvocationTargetException | InstantiationException | IllegalAccessException e) {
            System.out.println("Have fun with this stack trace ;)");
            e.printStackTrace();
            modules = new ArrayList<>();
        }

        try {
            JDABuilder builder = JDABuilder.createDefault(Config.getConfig().getString("bot.token"))
                    .disableCache(EnumSet.of(
                            CacheFlag.CLIENT_STATUS,
                            CacheFlag.ACTIVITY,
                            CacheFlag.EMOTE
                    ))
                            .enableCache(CacheFlag.VOICE_STATE);

            for (ListenerAdapter listener : PLUGINS) {
                builder.addEventListeners(listener);
            }

            jda = builder.build();

            CommandManager.registerSlashCommands();
            for (BuzzyModule module : modules) {
                module.onEnable(jda);
            }
            getActivity();
            LOGGER.info("BuzzBot finished loading!");
        } catch (LoginException e) {
            e.printStackTrace();
        }

        Listener.startProfanityFilter();
    }

    public static void getActivity() {
        if (Config.getConfig().getString("bot.activity").equalsIgnoreCase("playing")) {
            jda.getPresence().setActivity(Activity.playing(Config.getConfig().getString("bot.action")));
        } else if (Config.getConfig().getString("bot.activity").equalsIgnoreCase("watching")) {
            jda.getPresence().setActivity(Activity.watching(Config.getConfig().getString("bot.action")));
        } else if (Config.getConfig().getString("bot.activity").equalsIgnoreCase("competing")) {
            jda.getPresence().setActivity(Activity.competing(Config.getConfig().getString("bot.action")));
        } else if (Config.getConfig().getString("bot.activity").equalsIgnoreCase("listening")) {
            jda.getPresence().setActivity(Activity.listening(Config.getConfig().getString("bot.action")));
        } else if (Config.getConfig().getString("bot.activity").equalsIgnoreCase("debug")) {
            jda.getPresence().setActivity(Activity.listening("my prefix " + Config.getConfig().getString("bot.prefix")));
        } else {
            jda.getPresence().setActivity(Activity.playing("with myself!"));
        }
    }
}

