package net.nevet5gi.buzzbot;

import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.events.ReadyEvent;
import net.dv8tion.jda.api.events.guild.GuildJoinEvent;
import net.dv8tion.jda.api.events.interaction.SlashCommandEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageUpdateEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.nevet5gi.buzzbot.commands.utils.CommandManager;
import net.nevet5gi.buzzbot.database.SqlDB;
import net.nevet5gi.buzzbot.functions.ProfanityFilter;
import net.nevet5gi.buzzbot.objects.GuildData;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;

public class Listener extends ListenerAdapter {
    private static final Logger LOGGER = LoggerFactory.getLogger(Listener.class);
    private final CommandManager manager = new CommandManager();
    private static final ProfanityFilter profanityFilter = new ProfanityFilter();

    public static void startProfanityFilter() {
        profanityFilter.buildDictionaries();
    }

    @Override
    public void onReady(@Nonnull ReadyEvent event) {
        LOGGER.info("BuzzBot is ready: " + event.getJDA().getSelfUser().getAsTag());
    }

    @Override
    public void onGuildJoin(@Nonnull GuildJoinEvent event) {
        GuildData guild = new GuildData();
        guild.setName(event.getGuild().getName());
        guild.setId(event.getGuild().getIdLong());
        guild.setGroup("master");
        guild.setProfanityLevel(0);

        SqlDB sql = new SqlDB();
        sql.addGuild(guild);
        sql.close();
    }

    @Override
    public void onGuildMessageReceived(@Nonnull GuildMessageReceivedEvent event) {
        User user = event.getAuthor();
        String raw = event.getMessage().getContentRaw();

        if (user.isBot() || event.isWebhookMessage()) {
            return;
        }

        if (profanityFilter.containsProfanity(raw, "strong")) {
            SqlDB sql = new SqlDB();
            int profanityLevel = sql.getGuildData(event.getGuild().getIdLong()).getProfanityLevel();
            sql.close();

            switch (profanityLevel) {
                case 1 -> {
                    if (profanityFilter.containsProfanity(raw, "mild")) profanityFilter.handle(event);
                }
                case 2 -> {
                    if (profanityFilter.containsProfanity(raw, "moderate")) profanityFilter.handle(event);
                }
                case 3 -> profanityFilter.handle(event);
            }
        }

        if (raw.startsWith(Config.getConfig().getString("bot.prefix"))) {
            manager.handle(event);
        }
    }

    @Override
    public void onGuildMessageUpdate(@Nonnull GuildMessageUpdateEvent event) {
        String raw = event.getMessage().getContentRaw();

        if (profanityFilter.containsProfanity(raw, "strong")) {
            SqlDB sql = new SqlDB();
            int profanityLevel = sql.getGuildData(event.getGuild().getIdLong()).getProfanityLevel();
            sql.close();

            switch (profanityLevel) {
                case 1:
                    if (profanityFilter.containsProfanity(raw, "mild")) profanityFilter.handleEdit(event);
                    break;
                case 2:
                    if (profanityFilter.containsProfanity(raw, "moderate")) profanityFilter.handleEdit(event);
                    break;
                case 3:
                    profanityFilter.handleEdit(event);
                    break;
            }
        }
    }

    @Override
    public void onSlashCommand(@NotNull SlashCommandEvent event) {
        manager.handle(event);
    }
}
